import datetime

from qstrader import settings
from qstrader.strategy.base import AbstractStrategy
from qstrader.event import SignalEvent, EventType
from qstrader.compat import queue
from qstrader.trading_session import TradingSession


class BuyAndHoldStrategy(AbstractStrategy):
    """
    A testing strategy that simply purchases (longs) an asset
    upon first receipt of the relevant bar event and
    then holds until the completion of a backtest.
    """

    def __init__(
            self, ticker, events_queue,
            base_quantity=100000  # beware of fixed position sizer
    ):
        self.ticker = ticker
        self.events_queue = events_queue
        self.base_quantity = base_quantity
        self.bars = 0
        self.invested = False

    def calculate_signals(self, event):
        if (
                        event.type in [EventType.BAR, EventType.TICK, EventType.FXCMBAR] and  # added fxcm bar
                        event.ticker == self.ticker
        ):
            if not self.invested and self.bars % 15 == 0:
                signal = SignalEvent(
                    self.ticker, "BOT",
                    suggested_quantity=self.base_quantity
                )
                print("buy")
                self.events_queue.put(signal)
                self.invested = True

            elif self.invested and self.bars % 15 == 7:
                signal = SignalEvent(
                    self.ticker, "SLN",
                    suggested_quantity=self.base_quantity
                )
                print('sell')
                self.events_queue.put(signal)
                self.invested = False
            self.bars += 1


def run(config, testing, tickers, filename):
    # Backtest information
    title = ['Buy and Hold Example on %s' % tickers[0]]
    initial_equity = 100000.0
    start_date = datetime.datetime(2007, 1, 24)
    end_date = datetime.datetime(2017, 1, 23)

    # Use the Buy and Hold Strategy
    events_queue = queue.Queue()
    strategy = BuyAndHoldStrategy(tickers[0], events_queue)

    # Set up the backtest
    backtest = TradingSession(
        config, strategy, tickers,
        initial_equity, start_date, end_date,
        events_queue, title=title, end_session_time=datetime.datetime.now() + datetime.timedelta(0, 30))

    results = backtest.start_trading(testing=testing)
    return results


if __name__ == "__main__":
    # Configuration data
    testing = False
    config = settings.from_file(
        settings.DEFAULT_CONFIG_FILENAME, testing
    )
    tickers = ['EUR/JPY']
    filename = None
    run(config, testing, tickers, filename)
