import datetime

from .base import AbstractExecutionHandler
from ..event import (FillEvent, EventType)
from ..price_parser import PriceParser
from FXCMRestWarpper.orders import trade
from FXCMRestWarpper.auth import account
from FXCMRestWarpper.tables import snapshot


class FXCMLiveExecutionHandler(AbstractExecutionHandler):
    """
    TODO fix AUTH
    this is not finished yet
    """

    def __init__(self, events_queue, price_handler, auth, compliance=None):
        """
        Initialises the handler, setting the event queue
        as well as access to local pricing.
        Parameters:
        events_queue - The Queue of Event objects.
        """
        self.events_queue = events_queue
        self.price_handler = price_handler
        self.compliance = compliance
        self.auth = auth
        self.accounts = self._get_account()

    def execute_order(self, event):
        """
        Converts OrderEvents into FillEvents ,
        assume the fxcm account is a non hedging account,
        Parameters:
        event - An Event object with order information.
        TODO find fill price
        it can get fill price but not very well tested
        """
        if event.type == EventType.ORDER:
            # Obtain values from the OrderEvent
            timestamp = self.price_handler.get_last_timestamp(event.ticker)
            ticker = event.ticker
            action = event.action
            quantity = event.quantity

            # excute order
            fill_price = self._open_trade(ticker=ticker, action=action, quantity=quantity)
            exchange = None
            # Create the FillEvent and place on the events queue
            fill_event = FillEvent(
                timestamp, ticker,
                action, quantity,
                exchange, fill_price,
                commission=0
            )
            self.events_queue.put(fill_event)

            if self.compliance is not None:
                self.compliance.record_trade(fill_event)

    def _open_trade(self, action, quantity, ticker):
        """
        execute trade until the trade is executed
        TODO check response to see if the order execute
        """

        trades = trade.Trade(self.auth)
        exet = datetime.datetime.now()
        # using "AtMarket" so the order are excute guaranteed
        trades.openTrade(account_id=self.accounts[0], symbol=ticker, is_buy=action,
                         amount=quantity, order_type='AtMarket', time_in_force='IOC',rate=None)
        snap = snapshot.snapshot(self.auth)
        mylist = ['ClosedPosition', 'OpenPosition']
        rep = snap.getsnapshot(mylist)
        return self._check_trade_info(rep, exet)

    def _get_account(self):
        """
        get the accounts
        :return:
        a list of account
        """
        accounts = account.Account(self.auth)
        return accounts.get_account_number()

    def _check_trade_info(self, rep, exetime):
        """
        TODO ask david on executed trade
        assume the latest trade are on the first element
        for observation the latest position always on pos 0
        this logic compare times and always get the latest fill price
        """

        if rep['closed_positions'][0]['closeTime'] is None:
            print("no closed pos, using latest open pos")
            return rep['open_positions'][0]['close']
        elif rep['open_positions'][0]['time'] is None:
            print("no open pos, using latest closed pos")
            return rep['closed_positions'][0]['close']
        elif self._timeparser(rep['closed_positions'][0]['closeTime']) <\
                self._timeparser(rep['open_positions'][0]['time']) < exetime:
            print("using open pos")
            return rep['open_positions'][0]['close']
        elif self._timeparser(rep['closed_positions'][0]['closeTime']) >\
                self._timeparser(rep['open_positions'][0]['time']) < exetime:
            print("using closed pos")
            return rep['closed_positions'][0]['close']
        else:
            raise TypeError

    def _timeparser(self, time):
        month = time[:2]
        day = time[2:4]
        year = time[4:8]
        hour = time[8:10]
        mins = time[10:12]
        sec = time[-2:]
        return datetime.datetime(int(year), int(month), int(day), int(hour), int(mins), int(sec))