import pandas as pd

from FXCMRestWarpper.data import stream
from ..price_parser import PriceParser
from ..event import TickEvent
from .base import AbstractTickPriceHandler
import json
import datetime


class FXCMTickPriceHandler(AbstractTickPriceHandler):
    """
    """

    def __init__(self, events_queue, tickers, auth):
        self.price_event = None
        self.events_queue = events_queue
        self.continue_backtest = True
        self.tickers_lst = tickers
        self.tickers = {}
        self.socket = auth
        self.data = stream.stream(self.socket)
        self.disconnectcount = 0
        for ticker in self.tickers_lst:
            self.tickers[ticker] = {}
            self.data.subscribe(ticker)
            self.socket.socketIO.on(ticker, self.on_prices_update)
        # Adding the "on_price_update" function to Subscription
        # Registering the Subscription
        self.socket.socketIO.on('connect', self.con)
        self.socket.socketIO.on('disconnect', self.discon)
        self.socket.socketIO.on('reconnect', self.recon)
        # socket.socketIO.wait()

        # self.receive_events_thread = Thread(target=self._receive_events_thread(socket.socketIO))
        # self.receive_events_thread.daemon = True
        # self.receive_events_thread.start()

    def on_prices_update(self, msg):
        print(msg)
        tev = self._create_event(json.loads(msg))
        if self.price_event is not None:
            print("losing %s" % self.price_event)
        self.price_event = tev

    def discon(self):
        print("disCon")

        self.disconnectcount += 1
        if self.disconnectcount == 2:
            self.forcerecon()
        self.socket.socketIO.wait(5)

    def recon(self):
        print("recon")
        self.disconnectcount = 0
        self.data.subscribe('EUR/USD')
        self.socket.socketIO.on('EUR/USD', self.on_prices_update)

    def con(self):
        print("con")

    def forcerecon(self):
        """
        hard coded EURUSD
        :return:
        """
        for ticker in self.tickers_lst:
            self.data.subscribe(ticker)
            self.socket.socketIO.on(ticker, self.on_prices_update)
        self.socket.socketIO.on('connect', self.con)
        self.socket.socketIO.on('disconnect', self.discon)
        self.socket.socketIO.on('reconnect', self.recon)

    def _create_event(self, data):
        ticker = data["Symbol"]
        index = pd.to_datetime(datetime.datetime.fromtimestamp(float(data["Updated"])/1000))
        bid = PriceParser.parse(data["Rates"][0])
        ask = PriceParser.parse(data["Rates"][1])
        return TickEvent(ticker, index, bid, ask)

    def stream_next(self):
        """
        Place the next PriceEvent (BarEvent or TickEvent) onto the event queue.
        """
        if self.price_event is not None:
            self._store_event(self.price_event)
            self.events_queue.put(self.price_event)
            self.price_event = None

    async def listen(self):
        self.socket.socketIO.wait()
